
# q-expansion of newform 2178.4.a.c, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2178
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-2], [0], [-6], [-18], [0], [42], [30], [60], [186], [42], [16], [-74], [150], [-264], [-306], [-126], [-492], [6], [-524], [-390], [-984], [354], [1116], [-744], [1406], [1218], [200], [528], [-894], [408], [-1734], [1068], [3156], [2340], [1878], [-894], [-2126], [2360], [2640], [378], [-1380], [-1222], [-354], [3180], [3162], [-1856], [1128], [-3832], [-6504], [1762], [6918], [2184], [-108], [1932], [-4752], [6864], [-7902], [4206], [-5154], [4722], [1320], [-8490], [8244], [666], [-38], [-8490], [7400], [-10944], [9444], [9738], [11832], [-9144], [3728], [-6486], [1856], [8538], [9354], [11882], [4716], [-8892], [-11556], [-1298], [14568], [11342], [-8622], [5772], [11532], [-9996], [3726], [9272], [-1608], [9384], [-4088], [-3528], [18880], [-4824], [5226], [-10788], [2400], [18162], [-12084], [846], [-5496], [2454], [12564], [2986], [-10980], [-4998], [-19422], [840], [13914], [15726], [11088], [-5452], [20392], [14844], [1100], [-4326], [14622], [528], [14506], [-1164], [-10086], [21060], [19892], [-23154], [-830], [-14118], [7144], [-8742], [-22992], [-16776], [-13736], [6838], [-36330], [30732], [-18126], [29256], [15882], [-30150], [42672], [33306], [31232], [-19956], [18830], [6846], [20622], [-2646], [-24784], [30438], [15786], [-19200], [-13772], [20232], [34652], [9234], [-39906], [-36], [-26568], [-36966], [-49944], [45450], [-31638], [53700], [-22116], [50226], [-38288], [49770], [-39372], [53598], [39156], [-10726], [16896], [28944], [3584], [-2352], [-18516], [-39642], [-26178], [-5934], [63160], [-19428], [-5186], [-3702], [-37758], [42894], [69654], [-12148], [-39372], [47328], [-30026], [-44652], [27880], [-64278], [-30120], [-44400], [26760], [36074], [53730], [4224], [48726], [-8178], [-32902], [-39696], [-21444], [16518], [46856], [-12780], [-11826], [29356], [25236], [53382], [44224], [17424], [-59712], [37798], [3954], [-41742], [77922], [63678], [28014], [50706], [-51108], [67448], [37104], [-17286], [-31320], [-54162], [45810], [82920], [89966], [-65208], [67710], [-106626], [-8048], [95160], [-46190], [-11922], [66324], [32142], [36456], [-41592], [117664], [-114302], [36426], [11232], [58480], [-58104], [5916], [82344], [-99666], [11022], [38382], [9106], [48726], [48900], [-72682], [-18096], [67926], [-99948], [-112650], [89532], [53610], [-118206], [-10044], [86380], [-21642], [-18888], [101220], [48282], [21890], [-89404], [-98642], [94242], [-106656], [-105392], [-101256], [-11010], [130776], [-2640], [83880], [85408], [37416], [134322], [-39372], [49098], [16382], [-137742], [69712], [73962], [-45870], [-134748], [-65790], [39240], [138690], [83478], [-90848], [-40350], [43884], [-46056], [-5412], [121086], [-58818], [-67176], [37348], [-93458], [170616], [-166034], [75510], [45042], [110472], [18558], [155598], [83824], [48792], [-53448], [-82560], [-51192], [81250], [51042], [-113460], [-48134], [-27690], [79000], [-139878], [-64942], [-173252], [169144], [120744], [28242], [-24474], [109962], [149526], [136752], [103752], [-190824], [-73294], [-80982], [-169594], [157986], [-201742], [109236], [-184266], [-166120], [9858], [17004], [-96910], [69608], [-133896], [-43878], [-125496], [-42730], [-87174], [-25398], [-78882], [762], [-750], [21024], [125442], [114198], [-122466], [196242], [51882], [-21000], [-75260], [116278], [-28674], [-93558], [-151068], [111540], [-57184], [-68424], [-154854], [-110958], [5614], [-47448], [-89232], [-168384], [-173358], [-155748], [140994], [-241620], [83754], [59526], [14448], [-186], [-81704], [-94498], [114864], [-76290], [-63178], [-117978], [-147036], [-167384], [-205482], [-7188], [52614], [227124], [88976], [-188118], [-51618], [150252], [266262], [144324], [-175530], [273102], [-113434], [166782], [109996], [-24576], [225456], [-14346], [-245520], [160428], [-167616], [-58578], [5064], [150608], [184644], [-140352], [-45858], [-2562], [62154], [-300096], [-132874], [175314], [-222576], [-53754], [-31192], [308520]]
